
/*
  Updated from the sketch "onoff_getfromadafruit_iotcloud" by @avilmaru, september 15 2019
  See his project https://www.hackster.io/Avilmaru/smart-plug-with-arduino-mkr-wifi-1010-63cb25#toc-sketch-5
  And his Github data https://github.com/avilmaru/enchufe_inteligente_wifi
  Changes:
  - Using an MKR1000 board instead of an MKR1010
  - Using the WiFi101 library instead of the WiFiNINA library
  - Indication LEDs for "searching WiFi" and "connected to WiFi"
  - Automatically reconnecting WiFi when WiFi connection goes down  
  
  Alex Pikkert
  December 2019

  this sketch gets the last data of a "feed" from Adafruit IOT Cloud to switch a relay
  See www.io.adafruit.com how to set up an account with a standard feed for switching.
    
  The hardware circuit:
  - Arduino MKR 1000 board
  - 1 5V relay on pin D8. The operating voltage is 3.3V.
  - white LED via 1kOhm resistor on pin D6 to indicate "searching for WiFi" 
  - green LED via 1kOhm resistor on pin D7 to indicate "WiFi connected" 
  
 */

#include <ArduinoJson.h>  
#include <WiFi101.h>
#include "arduino_secrets.h"       // here are your secret log codes stored for wifi access and the IO Adafruit feed key
                                   // this file must be stored in your library directory in a separate directory.
char ssid[] = SECRET_SSID;         // your network SSID (name)
char pass[] = SECRET_PASS;         // your network password (use for WPA, or use as key for WEP)
int status = WL_IDLE_STATUS;
char server[] = "io.adafruit.com"; // name address for the Adafruit IOT Cloud

// Initialize the client library
WiFiClient client;
int search_Wifi = 6;
int found_Wifi =7;
int state = 0;
int relay_pin = 8;

void setup() {
  Serial.begin(9600);
  pinMode(relay_pin, OUTPUT);     // Relay
  pinMode(search_Wifi, OUTPUT);    // searching for wifi connection
  pinMode(found_Wifi, OUTPUT);     // wifi connected
  digitalWrite(relay_pin, LOW);
       
  ConnectToWifi();
}

void loop() {
   
    if (status != WL_CONNECTED) {ConnectToWifi();} // reconnect to WiFi when disconnected
        
    httpRequest();
    if (state == 1){digitalWrite(relay_pin, HIGH);}  //Turn on relay
    if (state == 2){digitalWrite(relay_pin, LOW);}   //Turn off relay
     
    Serial.println(state);
}

// this method makes a HTTP connection to the server:
void httpRequest() 
{
  // close any connection before send a new request.
  client.stop();
  Serial.println("Starting connection to server...");
  if (client.connect(server, 80)) 
  {
      Serial.println("connected to server");
      // Make a HTTP request:
      client.println("GET /api/v2/" IO_USERNAME "/feeds/on-off/data/last HTTP/1.1"); 
      
      client.println("Host: io.adafruit.com");  
      client.println("Connection: close");
      client.println("Content-Type: application/json");  
      client.println("X-AIO-Key: " IO_KEY); 
      
      // Terminate headers with a blank line
      if (client.println() == 0) {
        Serial.println(F("Failed to send request"));
        return;
      }
        
      // Check HTTP status
      char status[32] = {0};
      client.readBytesUntil('\r', status, sizeof(status));
      if (strcmp(status, "HTTP/1.1 200 OK") != 0) {
        Serial.print(F("Unexpected response: "));
        Serial.println(status);
        return;
      }
      
      // Skip HTTP headers
      char endOfHeaders[] = "\r\n\r\n";
      if (!client.find(endOfHeaders)) {
        Serial.println(F("Invalid response1"));
        return;
      }

      // Skip Adafruit headers
      char endOfHeaders2[] = "\r";
      if (!client.find(endOfHeaders2)) {
        Serial.println(F("Invalid response2"));
        return;
      }

      //Deserialize JSon
      const size_t capacity = JSON_OBJECT_SIZE(12) + 170;
      StaticJsonDocument<capacity> doc;

      DeserializationError error = deserializeJson(doc, client);
      if (error) {
        Serial.print(F("deserializeJson() failed: "));
        Serial.println(error.c_str());
        return;
      }
      
      const char* value = doc["value"];
    
      Serial.print("get data!:");
      Serial.println(value);

       if (strcmp(value, "relay_on") == 0){state = 1;}   
       if (strcmp(value, "relay_off") == 0){state = 2;}
        delay(2000);
}
}

void ConnectToWifi()
{
  // attempt to connect to Wifi network:
  digitalWrite(search_Wifi,HIGH);digitalWrite(found_Wifi,LOW);
  while (status != WL_CONNECTED) {
    Serial.print("Attempting to connect to SSID: ");
    Serial.println(ssid);
    // Connect to WPA/WPA2 network. Change this line if using open or WEP network:
    status = WiFi.begin(ssid, pass);
    // wait 10 seconds for connection:
    delay(10000);
  }
  Serial.println("Connected to wifi");
  digitalWrite(search_Wifi,LOW);digitalWrite(found_Wifi,HIGH);
  printWifiStatus();
}

void printWifiStatus() {
  // print the SSID of the network you're attached to:
  Serial.print("SSID: ");
  Serial.println(WiFi.SSID());

  // print your board's IP address:
  IPAddress ip = WiFi.localIP();
  Serial.print("IP Address: ");
  Serial.println(ip);

  // print the received signal strength:
  long rssi = WiFi.RSSI();
  Serial.print("signal strength (RSSI):");
  Serial.print(rssi);
  Serial.println(" dBm");

 
}
